@@ -8,6 +8,7 @@
 #include <proof_helpers/proof_allocators.h>
 
 void aws_string_destroy_harness() {
-    struct aws_string *str = nondet_bool() ? ensure_string_is_allocated_nondet_length() : NULL;
+    struct aws_string *str = ensure_string_is_allocated_nondet_length();
+    __CPROVER_assume(IMPLIES(str != NULL, aws_string_is_valid(str)));
     aws_string_destroy(str);
 }
