@@ -146,7 +146,12 @@ static int s_tls_common_tester_init(struct aws_allocator *allocator, struct tls_
     aws_atomic_store_ptr(&tester->stats_handler, NULL);
 
     tester->el_group = aws_event_loop_group_new_default(allocator, 0, NULL);
-    tester->resolver = aws_host_resolver_new_default(allocator, 1, tester->el_group, NULL);
+
+    struct aws_host_resolver_default_options resolver_options = {
+        .el_group = tester->el_group,
+        .max_entries = 1,
+    };
+    tester->resolver = aws_host_resolver_new_default(allocator, &resolver_options);
 
     return AWS_OP_SUCCESS;
 }
@@ -1196,11 +1201,11 @@ static void s_creation_callback_test_channel_creation_callback(
 
 static struct aws_event_loop *s_default_new_event_loop(
     struct aws_allocator *allocator,
-    aws_io_clock_fn *clock,
+    const struct aws_event_loop_options *options,
     void *user_data) {
 
     (void)user_data;
-    return aws_event_loop_new_default(allocator, clock);
+    return aws_event_loop_new_default_with_options(allocator, options);
 }
 
 static int s_statistic_test_clock_fn(uint64_t *timestamp) {
@@ -1224,7 +1229,12 @@ static int s_tls_common_tester_statistics_init(struct aws_allocator *allocator,
 
     tester->el_group =
         aws_event_loop_group_new(allocator, s_statistic_test_clock_fn, 1, s_default_new_event_loop, NULL, NULL);
-    tester->resolver = aws_host_resolver_new_default(allocator, 1, tester->el_group, NULL);
+
+    struct aws_host_resolver_default_options resolver_options = {
+        .el_group = tester->el_group,
+        .max_entries = 1,
+    };
+    tester->resolver = aws_host_resolver_new_default(allocator, &resolver_options);
 
     return AWS_OP_SUCCESS;
 }
